% checkBatchEffect() - Uses PCA, Kmeans, and Benjamini Hochberg correction
%            to determine if the provided data can be split into the expected 
%            groups to indicate a batch effect.  The analysis uses 
%            hypothesis testing of the null hypothesis that the 2
%            groups identified by kmeans are a random selection of the 2
%            anticipated groups.   PCA is used on subsets of the
%            (micro)RNAs provided in the matrix to create many possible
%            groupings with kmeans.   Multiple testing correction is done
%            at a specified FDR to determine if the groups identified by
%            PCA and Kmeans are more than random samples of the 2
%            anticipated groups.
%            Parameters that control the spefics of the run are contained
%            in config.m.
%
% Usage:
%  >>  [nTests nPassFDR FDR]=checkBatchEffect(FullSet, s1Ind, s2Ind, dimLabels, draw)
%
% Required Input:
%   FullSet - A matrix of the dimension you wish to analyze.  For example,
%           this could be a matric of microRNA expression values.   TCGA
%           expression matrix is expected where rows are microRNA
%           expressions and columns are samples.
%   s1Ind  - A list containing the 1-indexed indices of the columns that
%           contain the first set.   For example, the indices of the
%           samples that were run on Jan 1st.
%   s2Ind  - A list containing the indices of the columns that contain
%           samples from the second set.   For example this could contain
%           the indices for the samples that were run on Jan8th.
%           
%
% Optional Inputs:
%   dimLabels - If provided can be used in debug mode to determine which of
%             the dimenions are driving the analysis. 
%   draw      - If provided will instruct the script to draw plots at each
%             iteration of analysis.   This can be helpful to see how well
%             the groups are separating.
%   
%
% Outputs:
%   nTests    - The number of experiments that were evaluated
%   nPassFDR  - The number of experiments that were significant at the FDR
%   FDR       - False discovery rate
%
%
% Authors:
% Richard Corbett rcorbett@bcgsc.ca
% Inanc Birol ibirol@bcgsc.ca
% Canada's Michael Smith Genome Sciences Center
% May 27, 2011

function [nTests nPassFDR FDR] = checkBatchEffect(set1, s1Ind, s2Ind, labels)

    if(nargin < 3)
        disp('You must supply at least 3 arguments...');
        disp('The matrix of expression values')
        disp('The indices of which samples are from set 1');
        error('The indices of which samples are from set 2');
    end

    %load the operational parameters from the config file
    config;
    
    bigSet = set1'; % columns are uRNAs or RNA while rows are samples

    absMin = min(bigSet(bigSet>0));
    bigSet(bigSet<absMin) = absMin; %set 0s to non 0 minimum
    if(LOGIFY==1)
        bigSet = log10(bigSet);
    end
    
    %determine the order of importance of the dimensions
    means = mean(bigSet);
    medians = median(bigSet);
    spreads = abs(medians - means);
    [~, stdrIndex] = sort(spreads, 'descend');

    %These will be used for the FDR test at the end
    pValues = [];

    for n = DIM_ARRAY
        
        %get the subset based on most variant dims
        testSet = bigSet(:,stdrIndex(1:n));
       
        %Do the principal component analysis on the subset
        [coefs,scores,variances] = princomp(testSet);
       
        if(PLOT_LEVEL > 0)
            percentExplained = 100*variances/sum(variances);
            figure(3); clf; bar(percentExplained); title('Percent Explained');
        end
        
        disp(['n=' num2str(n)])
        
        % test the cross-product of the PCs to see if any are separated in
        % kmeans
        for i=1:(MAX_PC_TO_CHECK-1)
            for j = i+1:MAX_PC_TO_CHECK
                [idx,centers] = kmeans(scores(:,[i j]), K, 'replicates', N_REPLICATES);
                
                % a kmeans score to quantify the amount of separation
                sArray = silhouette(scores(:,[i j]),idx);
                
                %test the chance of getting this split as we did
                prob = testHyperGeometric(s1Ind, s2Ind, idx);
                pValues = [pValues; prob];
                
                if(PLOT_LEVEL==2 || (PLOT_LEVEL==1 && prob < 0.05))
                    makeplots(scores, idx, centers, s1Ind, s2Ind, i, j, n, sArray, prob)
                end
                
                %calculate the top contributors to each PC
                if(nargin >= 4 && ~isempty(labels))
                    [labels1 loads1 inds1] = getbigcontributors(coefs(:,i), labels, stdrIndex);
                    [labels2 loads2 inds2] = getbigcontributors(coefs(:,j), labels, stdrIndex);
                end
               
                disp(['nPC=' num2str(n) ', PCa=' num2str(i) ', PCb=' ...
                    num2str(j) ', s=' num2str(mean(sArray)), ...
                    ', p=' num2str(prob)]);

                if(prob < 1e-4)
                   takeABreak = 1; 
                end
            end
        end
    end
    
    %Benjamini-Hochberg
    for p = 1:length(FDR)
        [h, critP, adjP]=fdr_bh(pValues,FDR(p),'dep','yes');
    end
    nTests = length(h);
    nPassFDR = length(find(h==1));
    
    
% what are the names of the dims that are driving the separation on
% this PC?
% returns labels - the names of the top contributors
%         loads - the absolute value of the loadings
function [outLabels loads sortIndex] = getbigcontributors(coefs, inLabels, spreadOrder)
    [loads sortIndex] = sort(abs(coefs), 'descend');
    l = inLabels(spreadOrder+1); %reorder for spread
    outLabels = l(sortIndex); % reorder for loading

          
function makeplots(scores, idx, centers, s1Ind, s2Ind, i, j, n, sArray, prob)
    h = figure(1); clf; hold on;
    plot(scores(idx==1, i),scores(idx==1,j),'g.','MarkerSize',12)
    plot(scores(idx==2, i),scores(idx==2,j),'k.','MarkerSize',12)
    title(['KMeans Groups' 'nPC=' num2str(n) ', PCa=' num2str(i) ', PCb=' num2str(j) ' s=' num2str(mean(sArray))]);
    plot(centers(:,1),centers(:,2),'kx','MarkerSize',20,'LineWidth',2);
    plot(centers(:,1),centers(:,2),'ko','MarkerSize',20,'LineWidth',2);
    xlabel(['PC' num2str(i)])
    ylabel(['PC' num2str(j)])
    legend('Cluster 1', 'Cluster 2');

    h=figure(2); clf; hold on;
    plot(scores(s1Ind, i),scores(s1Ind,j),'b.','MarkerSize',12)
    plot(scores(s2Ind, i),scores(s2Ind,j),'r.','MarkerSize',12)
    xlabel(['PC' num2str(i)])
    ylabel(['PC' num2str(j)])
    title(['nPC=' num2str(n) ', PCa=' num2str(i) ', PCb=' ... 
        num2str(j) ', s=' num2str(mean(sArray)) ...
        ', p=' num2str(prob)]);

    h=figure(4); clf; hold on;
    for p = 1:size(scores, 1)
        if(~isempty(find(s1Ind==p)))
            if(idx(p) == 1)
               h1=plot(scores(p, i),scores(p,j),'^',...
                    'MarkerEdgeColor','k',...
                    'LineWidth',1,...
                    'MarkerSize',20, 'markerfacecolor', 'b');        
            else
               h2=plot(scores(p, i),scores(p,j),'o',...
                    'MarkerEdgeColor','k',...
                    'LineWidth',1,...
                    'MarkerSize',20, 'markerfacecolor', 'b');    
            end
        else
            if(idx(p) == 1)
                h3=plot(scores(p, i),scores(p,j),'^',...
                    'MarkerEdgeColor','k',...
                    'LineWidth',1,...
                    'MarkerSize',20, 'markerfacecolor', 'r');    
            else
                h4=plot(scores(p, i),scores(p,j),'o',...
                    'MarkerEdgeColor','k',...
                    'LineWidth',1,...
                    'MarkerSize',20, 'markerfacecolor', 'r');    
            end
        end
    end
    title(['nPC=' num2str(n) ', PCa=' num2str(i) ', PCb=' ... 
        num2str(j) ', s=' num2str(mean(sArray)) ...
        ', p=' num2str(prob)]);
    xlabel(['PC' num2str(i)])
    ylabel(['PC' num2str(j)])
    nm = sprintf('nDims_%d_pc1_%d_pc2_%d.png', n, i, j);
    

  
% what was the probabiltiy of getting as many of group 2 in this subset?
% s1Ind -> the indices of the first set
% s2Ind -> the indices of the second set
% idx -> the list telling which group each index ended up in
function pVal = testHyperGeometric(s1Ind, s2Ind, idx)

    g2 = find(idx==2);
    ng2 = length(g2);
    g1 = find(idx==1);
    ng1 = length(g1);

    nMatch = 0;
    for p = 1:length(s2Ind)
        for q = 1:ng2
            if(s2Ind(p)==g2(q))
               nMatch = nMatch+1; 
            end
        end
    end

    % X -> number of times we hit the target set
    % M -> population size
    % K -> number of samples in the target set
    % N -> number of samples drawn      

    pVal = hygecdf(nMatch,length(idx),length(s2Ind),ng2);

    %correct for the endpoint in the CDF.   We only want to include X
    %if we're on the upslope of the curve
    if(pVal > 0.5)
        pVal = 1-hygecdf(nMatch-1,length(idx),length(s2Ind),ng2);
    end

    

        