% findBatchEffect() - Uses PCA, Kmeans to find any potential batch effects
%            in the provided data
%            Parameters that control the spefics of the run are contained
%            in config.m.
%
% Usage:
%  >>  [nTests nPassFDR FDR]=findBatchEffect(FullSet, Labels, outPattern)
%
% Required Input:
%   FullSet - A matrix of the dimension you wish to analyze.  For example,
%           this could be a matric of microRNA expression values.   TCGA
%           expression matrix is expected where rows are microRNA
%           expressions and columns are samples.
%           
% Optional Inputs:
%   Labels  - If provided can be used in debug mode to determine which of
%             the dimensions are driving the analysis.   Also, when
%             sufficent separation is seen between groups, the sample names
%             can be drawn from this matrix and written to a file.
%   outPattern - If provided will instruct the scipt where to save the
%            output files.
%   
% Authors:
% Richard Corbett rcorbett@bcgsc.ca
% Inanc Birol ibirol@bcgsc.ca
% Canada's Michael Smith Genome Sciences Center
% May 27, 2011

function [] = findbatcheffect(set1, labels, outPattern)

    if(nargin < 1)
        disp('You must supply at least 1 arguments...');
        error('The matrix of expression values')
    end

    %load the operational parameters from the config file
    config;
    
    bigSet = set1'; % columns are uRNAs or RNA while rows are samples
        
    absMin = min(bigSet(bigSet>0));
    bigSet(bigSet<absMin) = absMin; %set 0s to non 0 minimum
    
    if(LOGIFY==1)
        bigSet = log10(bigSet);
    end
    
    %determine the order of importance of the dimensions
    means = mean(bigSet);
    medians = median(bigSet);
    spreads = abs(medians - means);
    [~, stdrIndex] = sort(spreads, 'descend');
    
    for n = DIM_ARRAY
        %Do the principal component analysis on the subset
        testSet = bigSet(:,stdrIndex(1:n));
        [coefs,scores,variances] = princomp(testSet);
        
        disp(['n=' num2str(n)])
        for i=1:(MAX_PC_TO_CHECK-1)
            for j = i+1:MAX_PC_TO_CHECK
                sils = zeros(1,MAX_K);
                for l=1:MAX_K
                    [idx,ctrs,sumd,D] = kmeans(scores(:,[i j]), l, 'replicates', N_REPLICATES);
                    s = silhouette(scores(:,[i j]),idx);
                    sils(l) = mean(s);
                end
                
                [best bestK] = max(sils);
                
                %redo at our best solution
                [idx,ctrs,sumd,D] = kmeans(scores(:,[i j]), bestK, 'replicates', N_REPLICATES);
                s = silhouette(scores(:,[i j]),idx);
                
                %calculate the top contributors to each PC
                if(~isempty(labels))
                    [labels1 loads1 inds1] = getbigcontributors(coefs(:,i), labels(:,1), stdrIndex);
                    [labels2 loads2 inds2] = getbigcontributors(coefs(:,j), labels(:,1), stdrIndex);
                end
                
                if( (mean(s) > DN_THRESHOLD && PLOT_LEVEL==1) || PLOT_LEVEL==2)
                    percentExplained = 100*variances/sum(variances);
                    figure(3); clf; bar(percentExplained); title('Percent Explained');
                    makeplots(scores, idx, ctrs, i, j, n, s, labels, bestK, outPattern);
                end
            end
        end
    end
        
    
% what are the names of the dims that are driving the separation on
% this PC?
% returns labels - the names of the top contributors
%         loads - the absolute value of the loadings
function [labels loads sortIndex] = getbigcontributors(coefs, dimLabels, spreadOrder)
    [loads sortIndex] = sort(abs(coefs), 'descend');
    l = dimLabels(spreadOrder+1); %reorder for spread
    labels = l(sortIndex); % reorder for loading
    
    
function makeplots(scores, idx, ctrs, i, j, n, s, labels, bestK, outPattern)
    h = figure(1); clf; hold on;
    cm = colormap;
    cmFact = round(64/bestK);
    for p=1:bestK
        plot(scores(idx==p, i),scores(idx==p,j),'.','MarkerSize',12, 'color', cm(1+(cmFact*(p-1)),:));
        plot(ctrs(:,1),ctrs(:,2),'kx','MarkerSize',20,'LineWidth',2);
        plot(ctrs(:,1),ctrs(:,2),'ko','MarkerSize',20,'LineWidth',2);
        xlabel(['PC' num2str(i)])
        ylabel(['PC' num2str(j)])
    end

    %display and save an image
    disp(['nPC=' num2str(n) ', PCa=' num2str(i) ', PCb=' ...
        num2str(j) ', s=' num2str(mean(s))]);
    nm = sprintf('%s_nDims_%d_pc1_%d_pc2_%d_sep_%1.2f.png', outPattern, n, i, j, mean(s));
    saveas(h, nm)

    %save a file showing the groups
    fnm = sprintf('%s_nDims_%d_pc1_%d_pc2_%d_sep_%1.2f.groups', outPattern, n, i, j, mean(s));
    fid = fopen(fnm, 'w');
    for iLine = 1:length(idx)
        fprintf(fid, '%s %d\n', char(labels(1,iLine+1)), idx(iLine));
    end
    fclose(fid);
        